/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#if ENABLE(GPU_PROCESS)
#include "RemoteMediaResourceManager.h"

#include "Decoder.h"
#include "HandleMessage.h"
#include "RemoteMediaResourceIdentifier.h"
#include "RemoteMediaResourceManagerMessages.h"
#include "SharedMemory.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/FrameLoaderTypes.h>
#include <WebCore/NetworkLoadMetrics.h>
#include <WebCore/ResourceError.h>
#include <WebCore/ResourceRequest.h>
#include <WebCore/ResourceResponse.h>

namespace Messages {

namespace RemoteMediaResourceManager {

void ResponseReceived::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::ShouldContinuePolicyCheck&&)>&& completionHandler)
{
    std::optional<WebCore::ShouldContinuePolicyCheck> shouldContinue;
    decoder >> shouldContinue;
    if (!shouldContinue) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*shouldContinue));
}

void ResponseReceived::cancelReply(CompletionHandler<void(WebCore::ShouldContinuePolicyCheck&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::ShouldContinuePolicyCheck>::create());
}

void ResponseReceived::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, WebCore::ShouldContinuePolicyCheck shouldContinue)
{
    encoder.get() << shouldContinue;
    connection.sendSyncReply(WTFMove(encoder));
}

void RedirectReceived::callReply(IPC::Decoder& decoder, CompletionHandler<void(WebCore::ResourceRequest&&)>&& completionHandler)
{
    std::optional<WebCore::ResourceRequest> returnRequest;
    decoder >> returnRequest;
    if (!returnRequest) {
        ASSERT_NOT_REACHED();
        cancelReply(WTFMove(completionHandler));
        return;
    }
    completionHandler(WTFMove(*returnRequest));
}

void RedirectReceived::cancelReply(CompletionHandler<void(WebCore::ResourceRequest&&)>&& completionHandler)
{
    completionHandler(IPC::AsyncReplyError<WebCore::ResourceRequest>::create());
}

void RedirectReceived::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection, const WebCore::ResourceRequest& returnRequest)
{
    encoder.get() << returnRequest;
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace RemoteMediaResourceManager

} // namespace Messages

namespace WebKit {

void RemoteMediaResourceManager::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::ResponseReceived::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaResourceManager::ResponseReceived>(connection, decoder, this, &RemoteMediaResourceManager::responseReceived);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::RedirectReceived::name())
        return IPC::handleMessageAsync<Messages::RemoteMediaResourceManager::RedirectReceived>(connection, decoder, this, &RemoteMediaResourceManager::redirectReceived);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::DataSent::name())
        return IPC::handleMessage<Messages::RemoteMediaResourceManager::DataSent>(connection, decoder, this, &RemoteMediaResourceManager::dataSent);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::DataReceived::name())
        return IPC::handleMessage<Messages::RemoteMediaResourceManager::DataReceived>(connection, decoder, this, &RemoteMediaResourceManager::dataReceived);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::AccessControlCheckFailed::name())
        return IPC::handleMessage<Messages::RemoteMediaResourceManager::AccessControlCheckFailed>(connection, decoder, this, &RemoteMediaResourceManager::accessControlCheckFailed);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::LoadFailed::name())
        return IPC::handleMessage<Messages::RemoteMediaResourceManager::LoadFailed>(connection, decoder, this, &RemoteMediaResourceManager::loadFailed);
    if (decoder.messageName() == Messages::RemoteMediaResourceManager::LoadFinished::name())
        return IPC::handleMessage<Messages::RemoteMediaResourceManager::LoadFinished>(connection, decoder, this, &RemoteMediaResourceManager::loadFinished);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit

#endif // ENABLE(GPU_PROCESS)
